==> poidsy-0.2.tgz: poidsy/processor.php <==
<?PHP

/* Poidsy 0.2 - http://chris.smith.name/projects/poidsy
 * Copyright (c) 2008 Chris Smith
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

 require_once(dirname(__FILE__) . '/discoverer.inc.php');
 require_once(dirname(__FILE__) . '/poster.inc.php');
 require_once(dirname(__FILE__) . '/urlbuilder.inc.php');

 /**
  * Redirects the user back to their original page.
  */
 function redirect() {
  if (defined('REDIRECT_URL')) {
   $url = REDIRECT_URL;
  } else if (isset($_SESSION['openid']['redirect'])) {
   $url = $_SESSION['openid']['redirect'];
  } else {
   $url = UrlBuilder::getCurrentURL();
  }

  doRedirect($url);
 }

 /**
  * Redirects the user to the specified URL.
  *
  * @param $url The URL to redirect the user to
  */ 
 function doRedirect($url) {
  header('Location: ' . $url);
  echo '<html><head><title>Redirecting</title></head><body>';
  echo '<p>Redirecting to <a href="', htmlentities($url), '">';
  echo htmlentities($url), '</a></p></body></html>';
  exit();
 }

 if (session_id() == "") {
  // No session - testing maybe?
  session_start();
 }

 // Process any openid_url form fields (compatability with 0.1)
 if (!defined('OPENID_URL') && isset($_POST['openid_url'])) {
  define('OPENID_URL', $_POST['openid_url']);
 }

 // Maximum number of requests to allow without a OPENID_THROTTLE_GAP second
 // gap between two of them
 if (!defined('OPENID_THROTTLE_NUM')) {
  define('OPENID_THROTTLE_NUM', 3);
 }

 // Time to require between requests before the request counter is reset 
 if (!defined('OPENID_THROTTLE_GAP')) {
  define('OPENID_THROTTLE_GAP', 30);
 }

 if (defined('OPENID_URL')) {
  // Initial authentication attempt (they just entered their identifier)

  // Check that they're not hammering us too much
  if (isset($_SESSION['openid']['requests'])) {
   $requests = $_SESSION['openid']['requests'];
  } else {
   $requests = array('lasttime' => 0, 'count' => 0);
  }

  if ($requests['lasttime'] < time() - OPENID_THROTTLE_GAP) {

   // Last request was a while ago, reset the timer
   $requests['count'] = 0;

  } else if ($requests['count'] > OPENID_THROTTLE_NUM) {

   // More than the legal number of requests 
   $_SESSION['openid']['error'] = 'You are trying to authenticate too often';
   redirect();

  }

  $requests['count']++;
  $requests['lasttime'] = time();

  try {
   $disc = new Discoverer(OPENID_URL);
  } catch (Exception $e) {
   $_SESSION['openid']['error'] = $e->getMessage();
   redirect();
  }

  $_SESSION['openid'] = array(
 	'identity' => $disc->getIdentity(), 
	'delegate' => $disc->getDelegate(),
	'validated' => false,
	'server' => $disc->getServer(),
	'nonce' => uniqid(microtime(true), true),
	'requests' => $requests, 
  );

  if ($disc->getServer() == null) {
   $_SESSION['openid']['error'] = 'Claimed identity is not a valid identifier';
   redirect();
  }

  $url = URLBuilder::buildImmediate($disc->getServer(), $disc->getDelegate(),
				    URLBuilder::getCurrentURL());

  doRedirect($url);
 } else if (isset($_REQUEST['openid_mode'])) {
  if ($_REQUEST['openid_nonce'] != $_SESSION['openid']['nonce']) {
   $_SESSION['openid']['error'] = 'Nonce doesn\'t match - possible replay attack';
   redirect();
  } else {
   $_SESSION['openid']['nonce'] = uniqid(microtime(true), true);
  }

  switch ($_REQUEST['openid_mode']) {
   case 'id_res':
    if (isset($_REQUEST['openid_identity'])) {
     if ($_REQUEST['openid_identity'] != $_SESSION['openid']['delegate']) {
      $_SESSION['openid']['error'] = 'Identity provider validated wrong '
				   . 'identity. Expected it to validate '
                                   . $_SESSION['openid']['delegate']
                                   . ' but it validated '
                                   . $_REQUEST['openid_identity'];
      redirect();
     }

     $url = URLBuilder::buildAuth($_REQUEST);

     try {
      $data = Poster::post($_SESSION['openid']['server'], $url);
     } catch (Exception $ex) {
      $_SESSION['openid']['error'] = 'Error authenticating response: ' 
                                   . $ex->getMessage();
      redirect();
     }

     foreach (explode("\n", $data) as $line) {
      if (substr($line, 0, 9) == 'is_valid:') {
       $valid = (boolean) substr($line, 9);
      }
     }

     $_SESSION['openid']['validated'] = $valid;

     if (!$valid) {
      $_SESSION['openid']['error'] = 'Provider didn\'t authenticate response';
     }

     redirect();

    } else if (isset($_REQUEST['openid_user_setup_url'])) {
     $url = URLBuilder::buildSetup($_REQUEST['openid_user_setup_url'], 
                                   $_SESSION['openid']['delegate'],
                                   URLBuilder::getCurrentURL());

     doRedirect($url);
    }
    break;
   case 'cancel':
    $_SESSION['openid']['error'] = 'Provider cancelled the authentication '
                                 . 'attempt';

    redirect();
    break; 
   case 'error':
    $_SESSION['openid']['error'] = 'Provider error: '
                                 . $_REQUEST['openid_error'];

    redirect();
    break;
  }
 }


?>

==> poidsy-0.3.tgz: poidsy/processor.php <==
<?PHP

/* Poidsy 0.3 - http://chris.smith.name/projects/poidsy
 * Copyright (c) 2008 Chris Smith
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

 require_once(dirname(__FILE__) . '/discoverer.inc.php');
 require_once(dirname(__FILE__) . '/poster.inc.php');
 require_once(dirname(__FILE__) . '/sreg.inc.php');
 require_once(dirname(__FILE__) . '/urlbuilder.inc.php');
 require_once(dirname(__FILE__) . '/keymanager.inc.php');

 /**
  * Redirects the user back to their original page.
  */
 function redirect() {
  if (defined('REDIRECT_URL')) {
   $url = REDIRECT_URL;
  } else if (isset($_SESSION['openid']['redirect'])) {
   $url = $_SESSION['openid']['redirect'];
  } else {
   $url = UrlBuilder::getCurrentURL();
  }

  doRedirect($url);
 }

 /**
  * Redirects the user to the specified URL.
  *
  * @param $url The URL to redirect the user to
  */ 
 function doRedirect($url) {
  header('Location: ' . $url);
  echo '<html><head><title>Redirecting</title></head><body>';
  echo '<p>Redirecting to <a href="', htmlentities($url), '">';
  echo htmlentities($url), '</a></p></body></html>';
  exit();
 }

 if (session_id() == "") {
  // No session - testing maybe?
  session_start();
 }

 // Process any openid_url form fields (compatability with 0.1)
 if (!defined('OPENID_URL') && isset($_POST['openid_url'])) {
  define('OPENID_URL', $_POST['openid_url']);
 }

 // Maximum number of requests to allow without a OPENID_THROTTLE_GAP second
 // gap between two of them
 if (!defined('OPENID_THROTTLE_NUM')) {
  define('OPENID_THROTTLE_NUM', 3);
 }

 // Time to require between requests before the request counter is reset 
 if (!defined('OPENID_THROTTLE_GAP')) {
  define('OPENID_THROTTLE_GAP', 30);
 }

 // Whether or not to use the key manager
 if (defined('OPENID_NOKEYMANAGER')) {
  define('KEYMANAGER', false);
 } else {
  define('KEYMANAGER', KeyManager::isSupported());
 }

 if (defined('OPENID_URL')) {
  // Initial authentication attempt (they just entered their identifier)

  // Check that they're not hammering us too much
  if (isset($_SESSION['openid']['requests'])) {
   $requests = $_SESSION['openid']['requests'];
  } else {
   $requests = array('lasttime' => 0, 'count' => 0);
  }

  if ($requests['lasttime'] < time() - OPENID_THROTTLE_GAP) {

   // Last request was a while ago, reset the timer
   $requests['count'] = 0;

  } else if ($requests['count'] > OPENID_THROTTLE_NUM) {

   // More than the legal number of requests 
   $_SESSION['openid']['error'] = 'You are trying to authenticate too often';
   $_SESSION['openid']['errorcode'] = 'throttled';
   redirect();

  }

  $requests['count']++;
  $requests['lasttime'] = time();

  try {
   $disc = new Discoverer(OPENID_URL);
  } catch (Exception $e) {
   $_SESSION['openid']['error'] = $e->getMessage();
   $_SESSION['openid']['errorcode'] = 'discovery';
   redirect();
  }

  $_SESSION['openid'] = array(
 	'identity' => $disc->getIdentity(), 
	'delegate' => $disc->getDelegate(),
	'validated' => false,
	'server' => $disc->getServer(),
	'nonce' => uniqid(microtime(true), true),
	'requests' => $requests, 
  );

  if ($disc->getServer() == null) {
   $_SESSION['openid']['error'] = 'Claimed identity is not a valid identifier';
   $_SESSION['openid']['errorcode'] = 'notvalid';
   redirect();
  }

  if (KEYMANAGER) {
   if (!KeyManager::hasKey($disc->getServer())) {
    KeyManager::associate($disc->getServer());
   }

   $handle = KeyManager::getHandle($disc->getServer());
  } else {
   $handle = null;
  }

  if (defined('OPENID_IMMEDIATE')) {
   $url = URLBuilder::buildImmediate($disc->getServer(), $disc->getDelegate(),
 				    URLBuilder::getCurrentURL(), $handle);
  } else {
   $url = URLBuilder::buildSetup($disc->getServer(), $disc->getDelegate(),
                                   URLBuilder::getCurrentURL(), $handle);
  }

  doRedirect($url);
 } else if (isset($_REQUEST['openid_mode'])) {
  if ($_REQUEST['openid_nonce'] != $_SESSION['openid']['nonce']) {
   $_SESSION['openid']['error'] = 'Nonce doesn\'t match - possible replay attack';
   $_SESSION['openid']['errorcode'] = 'nonce';
   redirect();
  } else {
   $_SESSION['openid']['nonce'] = uniqid(microtime(true), true);
  }

  if (KEYMANAGER && isset($_REQUEST['openid_invalidate_handle'])) {
   KeyManager::removeKey($_SESSION['openid']['server'], $_REQUEST['openid_invalidate_handle']);
  }

  switch ($_REQUEST['openid_mode']) {
   case 'id_res':
    if (isset($_REQUEST['openid_identity'])) {
     if ($_REQUEST['openid_identity'] != $_SESSION['openid']['delegate']) {
      $_SESSION['openid']['error'] = 'Identity provider validated wrong '
				   . 'identity. Expected it to validate '
                                   . $_SESSION['openid']['delegate']
                                   . ' but it validated '
                                   . $_REQUEST['openid_identity'];
      $_SESSION['openid']['errocode'] = 'diffid';
      redirect();
     }

     if (KEYMANAGER) {
      $dumbauth = false;

      try {
       $valid = KeyManager::authenticate($_SESSION['openid']['server'],
					 $_REQUEST);
      } catch (Exception $ex) {
       $dumbauth = true;
      }
     } else {
      $dumbauth = true;
     }

     if ($dumbauth) {
      $url = URLBuilder::buildAuth($_REQUEST);

      try {
       $data = Poster::post($_SESSION['openid']['server'], $url);
      } catch (Exception $ex) {
       $_SESSION['openid']['error'] = 'Error authenticating response: ' 
                                   . $ex->getMessage();
       $_SESSION['openid']['errorcode'] = 'autherror';
       redirect();
      }

      $valid = false;
      foreach (explode("\n", $data) as $line) {
       if (substr($line, 0, 9) == 'is_valid:') {
        $valid = (boolean) substr($line, 9);
       }
      }
     }

     $_SESSION['openid']['validated'] = $valid;

     if (!$valid) {
      $_SESSION['openid']['error'] = 'Provider didn\'t authenticate response';
      $_SESSION['openid']['errorcode'] = 'noauth';
     }

     parseSRegResponse();
     redirect();

    } else if (isset($_REQUEST['openid_user_setup_url'])) {
     if (OPENID_IMMEDIATE) {
      $_SESSION['openid']['error'] = 'Couldn\'t perform immediate auth';
      $_SESSION['openid']['errorcode'] = 'noimmediate';
      redirect();
     }

     if (KEYMANAGER) {
      $handle = KeyManager::getHandle($_SESSION['openid']['server']);
     } else {
      $handle = null;
     }

     $url = URLBuilder::buildSetup($_REQUEST['openid_user_setup_url'], 
                                   $_SESSION['openid']['delegate'],
                                   URLBuilder::getCurrentURL(), $handle);

     doRedirect($url);
    }
    break;
   case 'cancel':
    $_SESSION['openid']['error'] = 'Provider cancelled the authentication '
                                 . 'attempt';
    $_SESSION['openid']['errorcode'] = 'cancelled';
    redirect();
    break; 
   case 'error':
    $_SESSION['openid']['error'] = 'Provider error: '
                                 . $_REQUEST['openid_error'];
    $_SESSION['openid']['errorcode'] = 'perror';
    redirect();
    break;
  }
 }


?>
